#!/bin/sh
#

##!!
. /etc/service.subr

prog_dir=`dirname \`realpath $0\``
base_dir=/mnt/DroboFS/Shares/DroboApps/elephantdrive
config_dir=$base_dir/config

name="elephantdrive"
framework_version="2.1"
description="Secure cloud backup solution optimized for the Drobo platform"
depends=""
webui="WebUI"
pidfile=/tmp/DroboApps/elephantdrive/pid.txt
daemon=$base_dir/elephantdrive

errorfile=/tmp/DroboApps/elephantdrive/error.txt
statusfile=/tmp/DroboApps/elephantdrive/status.txt
edstatusfile=$base_dir/var/lib/elephantdrive/status

version="3.6.8"


start()
{
	mkdir -p /tmp/DroboApps/elephantdrive
	# delete edstatufile before starting daemon to delete previous status
	rm -f $edstatusfile
	rm -f $errorfile

	if [ ! -f $config_dir/elephantdrive.config ]; then
		mkdir -p $config_dir
		touch $config_dir/elephantdrive.config
	fi

	start-stop-daemon -S -b -m -p $pidfile -q -x env LD_LIBRARY_PATH="$base_dir" $daemon -- -l -c $config_dir \
    "MonitorConfigChanges=true" \
    "AppRoot=$base_dir" \
		"SmbConfFile=/mnt/DroboFS/System/samba/conf/smb.conf" \
		"CreateStatusFile=yes" \
		"SmbIgnoredDirs=/mnt/DroboFS/Shares/DroboApps"
	if [ -z $(pidof elephantdrive) ]; then
		echo 1 > $errorfile
		echo "Application starting error" > $edstatusfile
	fi
	sleep 1
	update_status
}

update_status()
{

	# wait until file appears
	i=60

	while [ ! -f $edstatusfile ] 
	do
		sleep 1
		i=$((i-1))

		if [ $i -eq 0 ] 
		then
			break
		fi
	done

	# if we don't have file here. throw error into status and return
	if [ ! -f $edstatusfile ] 
	then
		echo 1 > "${errorfile}"
		echo "Unknown status" > $statusfile
		return
	fi

	status1=`head -n 1 $edstatusfile`
	status2=`head -n 2 $edstatusfile | tail -n 1`

	case "$status1" in

		"login successful")
			echo 0 > "${errorfile}"
			echo "Application is running" > $statusfile
		;;

		"login failed")

			## 2nd level case
			echo 1 > "${errorfile}"
			case "$status2" in

				"ERROR_VAULT_LOGIN_USER_NOT_FOUND")
					echo "Username not found" > $statusfile
				;;

				"ERROR_VAULT_LOGIN_INCORRECT_PASSWORD")
					echo "Incorrect password" > $statusfile
				;;

				"ERROR_VAULT_ACCOUNT_SUSPENDED")
					echo "Account has been suspended" > $statusfile
				;;

				"ERROR_VAULT_ACCOUNT_CANCELLED")
					echo "Account has been cancelled" > $statusfile
				;;

				"Empty username")
					echo "Empty username. Configuration requires" > $statusfile
				;;

				*)
					echo "Loading ..." > $statusfile
				;;

			esac

		;;

		*)
			echo 1 > "${errorfile}"
			echo "Unknown status" > $statusfile
		;;
	esac

}

case "$1" in
	update_status)
		update_status
		exit $?
		;;
esac

main "$@"
